@@ -30,6 +30,7 @@
 
 #include "nautilus-file.h"
 #include "nautilus-file-utilities.h"
+#include "nautilus-scheme.h"
 #include "nautilus-search-engine.h"
 #include "nautilus-search-provider.h"
 #include "nautilus-ui-utilities.h"
@@ -66,7 +67,7 @@ struct _NautilusShellSearchProvider
 
 G_DEFINE_TYPE (NautilusShellSearchProvider, nautilus_shell_search_provider, G_TYPE_OBJECT)
 
-static gchar *
+static const char *
 get_display_name (NautilusShellSearchProvider *self,
                   NautilusFile                *file)
 {
@@ -82,7 +83,7 @@ get_display_name (NautilusShellSearchProvider *self,
 
     if (bookmark)
     {
-        return g_strdup (nautilus_bookmark_get_name (bookmark));
+        return nautilus_bookmark_get_name (bookmark);
     }
     else
     {
@@ -119,6 +120,7 @@ pending_search_free (PendingSearch *search)
 {
     g_hash_table_destroy (search->hits);
     g_clear_object (&search->query);
+    g_signal_handlers_disconnect_by_data (G_OBJECT (search->engine), search);
     g_clear_object (&search->engine);
     g_clear_object (&search->invocation);
 
@@ -148,19 +150,28 @@ cancel_current_search (NautilusShellSearchProvider *self)
 {
     if (self->current_search != NULL)
     {
+        NautilusSearchProvider *engine;
+
         g_debug ("*** Cancel current search");
-        nautilus_search_provider_stop (NAUTILUS_SEARCH_PROVIDER (self->current_search->engine));
+
+        engine = NAUTILUS_SEARCH_PROVIDER (self->current_search->engine);
+        /* The finish signal may be emitted during the call to nautilus_search_provider_stop
+         * which causes shell_search_provider to free the engine. Increase
+         * the ref count to prevent use after free issues.
+         */
+        g_object_ref (engine);
+        nautilus_search_provider_stop (engine);
+        g_object_unref (engine);
     }
 }
 
 static void
 cancel_current_search_ignoring_partial_results (NautilusShellSearchProvider *self)
 {
+    cancel_current_search (self);
+
     if (self->current_search != NULL)
     {
-        cancel_current_search (self);
-        g_signal_handlers_disconnect_by_data (G_OBJECT (self->current_search->engine),
-                                              self->current_search);
         pending_search_finish (self->current_search, self->current_search->invocation,
                                g_variant_new ("(as)", NULL));
     }
@@ -332,7 +343,7 @@ search_add_volumes_and_bookmarks (PendingSearch *search)
     g_free (uri);
 
     /* trash */
-    candidate = search_hit_candidate_new ("trash:///", _("Trash"));
+    candidate = search_hit_candidate_new (SCHEME_TRASH ":///", _("Trash"));
     candidates = g_list_prepend (candidates, candidate);
 
     /* now add mounts */
@@ -445,15 +456,15 @@ static NautilusQuery *
 shell_query_new (gchar **terms)
 {
     NautilusQuery *query;
-    g_autoptr (GFile) home = NULL;
     g_autofree gchar *terms_joined = NULL;
 
     terms_joined = g_strjoinv (" ", terms);
-    home = g_file_new_for_path (g_get_home_dir ());
 
     query = nautilus_query_new ();
     nautilus_query_set_text (query, terms_joined);
-    nautilus_query_set_location (query, home);
+    /* Global search is not limited by location. */
+    nautilus_query_set_location (query, NULL);
+    nautilus_query_set_recursive (query, NAUTILUS_QUERY_RECURSIVE_INDEXED_ONLY);
 
     return query;
 }
@@ -477,7 +488,6 @@ execute_search (NautilusShellSearchProvider  *self,
     }
 
     query = shell_query_new (terms);
-    nautilus_query_set_recursive (query, NAUTILUS_QUERY_RECURSIVE_INDEXED_ONLY);
     nautilus_query_set_show_hidden_files (query, FALSE);
 
     pending_search = g_slice_new0 (PendingSearch);
@@ -502,6 +512,7 @@ execute_search (NautilusShellSearchProvider  *self,
 
     /* start searching */
     g_debug ("*** Search engine search started");
+    nautilus_search_engine_enable_recent (pending_search->engine);
     nautilus_search_provider_set_query (NAUTILUS_SEARCH_PROVIDER (pending_search->engine),
                                         query);
     nautilus_search_provider_start (NAUTILUS_SEARCH_PROVIDER (pending_search->engine));
@@ -631,20 +642,22 @@ result_list_attributes_ready_cb (GList    *file_list,
     ResultMetasData *data = user_data;
     GVariantBuilder meta;
     NautilusFile *file;
-    GFile *file_location;
     GList *l;
-    gchar *uri, *display_name;
+    gchar *uri;
+    const char *display_name;
     gchar *path, *description;
-    gchar *thumbnail_path;
     GIcon *gicon;
     GFile *location;
     GVariant *meta_variant;
     gint icon_scale;
 
-    icon_scale = gdk_monitor_get_scale_factor (gdk_display_get_monitor (gdk_display_get_default (), 0));
+    icon_scale = gdk_monitor_get_scale_factor (g_list_model_get_item (gdk_display_get_monitors (gdk_display_get_default ()), 0));
 
     for (l = file_list; l != NULL; l = l->next)
     {
+        g_autoptr (GFile) file_location = NULL;
+        g_autoptr (GVariant) icon_variant = NULL;
+
         file = l->data;
         g_variant_builder_init (&meta, G_VARIANT_TYPE ("a{sv}"));
 
@@ -663,14 +676,13 @@ result_list_attributes_ready_cb (GList    *file_list,
                                "description", g_variant_new_string (description ? description : uri));
 
         gicon = NULL;
-        thumbnail_path = nautilus_file_get_thumbnail_path (file);
 
+        const char *thumbnail_path = nautilus_file_get_thumbnail_path (file);
         if (thumbnail_path != NULL)
         {
             location = g_file_new_for_path (thumbnail_path);
             gicon = g_file_icon_new (location);
 
-            g_free (thumbnail_path);
             g_object_unref (location);
         }
         else
@@ -680,20 +692,20 @@ result_list_attributes_ready_cb (GList    *file_list,
 
         if (gicon == NULL)
         {
-            gicon = G_ICON (nautilus_file_get_icon_pixbuf (file, 128, TRUE,
-                                                           icon_scale,
-                                                           NAUTILUS_FILE_ICON_FLAGS_USE_THUMBNAILS));
+            gicon = G_ICON (nautilus_file_get_icon_texture (file, 128,
+                                                            icon_scale,
+                                                            NAUTILUS_FILE_ICON_FLAGS_USE_THUMBNAILS));
         }
 
+        icon_variant = g_icon_serialize (gicon);
         g_variant_builder_add (&meta, "{sv}",
-                               "icon", g_icon_serialize (gicon));
+                               "icon", icon_variant);
         g_object_unref (gicon);
 
         meta_variant = g_variant_builder_end (&meta);
         g_hash_table_insert (data->self->metas_cache,
                              g_strdup (uri), g_variant_ref_sink (meta_variant));
 
-        g_free (display_name);
         g_free (path);
         g_free (description);
         g_free (uri);
@@ -761,12 +773,13 @@ typedef struct
 } ShowURIData;
 
 static void
-show_uri_callback (gboolean res,
-                   gpointer user_data)
+show_uri_callback (GObject      *source_object,
+                   GAsyncResult *result,
+                   gpointer      user_data)
 {
     ShowURIData *data = user_data;
 
-    if (!res)
+    if (!gtk_show_uri_full_finish (NULL, result, NULL))
     {
         g_application_open (g_application_get_default (), &data->file, 1, "");
     }
@@ -786,16 +799,13 @@ handle_activate_result (NautilusShellSearchProvider2  *skeleton,
                         gpointer                       user_data)
 {
     ShowURIData *data;
-    gboolean res;
 
     data = g_new (ShowURIData, 1);
     data->file = g_file_new_for_uri (result);
     data->skeleton = skeleton;
     data->invocation = invocation;
 
-    res = gtk_show_uri_on_window (NULL, result, timestamp, NULL);
-
-    show_uri_callback (res, data);
+    gtk_show_uri_full (NULL, result, timestamp, NULL, show_uri_callback, data);
 
     return TRUE;
 }
@@ -809,24 +819,6 @@ handle_launch_search (NautilusShellSearchProvider2  *skeleton,
 {
     GApplication *app = g_application_get_default ();
     g_autoptr (NautilusQuery) query = shell_query_new (terms);
-    NautilusQueryRecursive recursive = location_settings_search_get_recursive ();
-
-    /*
-     * If no recursive search is enabled, we still want to be able to
-     * show the same results we presented in the overview when nautilus
-     * is explicitly launched to access to more results, and thus we perform
-     * a query showing results coming from index-based search engines.
-     * Otherwise we respect the global setting for recursivity.
-     */
-    if (recursive == NAUTILUS_QUERY_RECURSIVE_NEVER)
-    {
-        nautilus_query_set_recursive (query,
-                                      NAUTILUS_QUERY_RECURSIVE_INDEXED_ONLY);
-    }
-    else
-    {
-        nautilus_query_set_recursive (query, recursive);
-    }
 
     nautilus_application_search (NAUTILUS_APPLICATION (app), query);
 
