@@ -1410,14 +1410,15 @@ nautilus_window_show_operation_notification (NautilusWindow *window,
     if (!is_current_location)
     {
         g_autoptr (NautilusFile) folder = NULL;
-        g_autofree gchar *button_label = NULL;
+        g_autoptr (GString) button_label = g_string_new ("");
         GVariant *target;
 
         target = g_variant_new_take_string (g_file_get_uri (folder_to_open));
         folder = nautilus_file_get (folder_to_open);
-        button_label = g_strdup_printf (_("Open %s"),
-                                        nautilus_file_get_display_name (folder));
-        adw_toast_set_button_label (toast, button_label);
+        g_string_printf (button_label, _("Open %s"), nautilus_file_get_display_name (folder));
+        /* Need to escape mnemonics since it's a button. */
+        g_string_replace (button_label, "_", "__", -1);
+        adw_toast_set_button_label (toast, button_label->str);
         adw_toast_set_action_name (toast, "win.open-location");
         adw_toast_set_action_target_value (toast, target);
     }
@@ -2100,10 +2101,11 @@ nautilus_window_unexport_handle (NautilusWindow *window)
     if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (GTK_WIDGET (window))))
     {
         GdkSurface *gdk_surface = gtk_native_get_surface (GTK_NATIVE (window));
-        if (GDK_IS_WAYLAND_TOPLEVEL (gdk_surface))
+        if (GDK_IS_WAYLAND_TOPLEVEL (gdk_surface) &&
+            g_str_has_prefix (window->export_handle, "wayland:"))
         {
             gdk_wayland_toplevel_drop_exported_handle (GDK_WAYLAND_TOPLEVEL (gdk_surface),
-                                                       window->export_handle);
+                                                       window->export_handle + strlen ("wayland:"));
         }
     }
 #endif
